﻿import java.util.ArrayList;
import java.util.List;

public class Hand
{
    final int NoRepeatedCardValue = -1;
    final int ACE = 1;
    List<Card> cards;
    

    public Hand(List<Card> cards)
    {
        this.cards = cards;
    }

    public Ranking FindBestRanking()
    {
        OrderCardsByNumber();

        if (ContainsRoyalStraightFlush())
            return Ranking.RoyalStraightFlush;
        if (ContainsStraightFlush())
            return Ranking.StraightFlush;
        if (ContainsFourOfAKind())
            return Ranking.FourOfAKind;
        if (ContainsFullHouse())
            return Ranking.FullHouse;
        if (ContainsRoyalFlush())
            return Ranking.RoyalFlush;
        if (ContainsFlush())
            return Ranking.Flush;
        if (IsStraightHand())
            return Ranking.Straight;
        if (ContainsThreeOfAKind())
            return Ranking.ThreeOfAKind;
        if (ContainsTwoPair())
            return Ranking.TwoPair;
        if (ContainsPair())
            return Ranking.Pair;

        return Ranking.HighCard;
    }

    public Winner PlayAgainst(Hand secondHand)
    {
        if (FindBestRanking().hasHigherValueThan(
        		secondHand.FindBestRanking()))
        {
            return Winner.First;
        }

        if (FindBestRanking().hasLowerValueThan(
        		secondHand.FindBestRanking()))
        {
            return Winner.Second;
        }

        return Winner.Tie;
    }

    private List<Card> GetCards()
    {
        return cards;
    }

    private Card GetCardAt(int index)
    {
        return cards.get(index);
    }

    private int GetCardNumberAt(int index)
    {
        return cards.get(index).GetNumber();
    }

    private Card.Suit GetCardSuitAt(int index)
    {
        return cards.get(index).GetSuit();
    }

    private void SetCards(List<Card> cards)
    {
        this.cards = cards;
    }

    private int GetNumCards()
    {
        return cards.size();
    }

    private void RemoveCard(int value)
    {
        for(int i = 0;i< cards.size();i++)
        {
            if (cards.get(i).GetNumber() == value)
            {
                cards.remove(i);        
            }
        }
    }

    private int GetNoRepeatedCardValue()
    {
        return NoRepeatedCardValue;
    }

    private boolean ContainsAce()
    {
        int index = 0;
        boolean aceFound = false;

        while (!aceFound && index < GetNumCards())
        {
            if (GetCardNumberAt(index) == ACE)
            {
                aceFound = true;
            }
            else
            {
                index++;
            }
        }

        return aceFound;
    }

    private int GetRepeatedCard(int numberOfTimes)
    {
        boolean isARepeatedCard = false;
        int repeatedCard = NoRepeatedCardValue;
        int i = 0;

        while(!isARepeatedCard && i< GetNumCards())
        {
            int cardCounter = 1;
            int j = i + 1;

            while (!isARepeatedCard && j < GetNumCards())
            {
                int currentCard = GetCardNumberAt(i);
                int nextCard = GetCardNumberAt(j);

                if (currentCard == nextCard)
                {
                    cardCounter++;
                }

                if (cardCounter == numberOfTimes)
                {
                    isARepeatedCard = true;
                    repeatedCard = currentCard;
                }
                else
                {
                    j++;
                }
            }

            i++;
        }

        return repeatedCard;
    }

    private boolean ContainsFullHouse()
    {
        Hand auxHand = new Hand(new ArrayList<Card>(GetCards()));
        int repeatedCard = auxHand.GetRepeatedCard(2);

        if (repeatedCard != GetNoRepeatedCardValue())
        {
            auxHand.RemoveCard(repeatedCard);
            auxHand.RemoveCard(repeatedCard);

            if (auxHand.GetRepeatedCard(3) != GetNoRepeatedCardValue())
            {
                return true;
            }
        }

        return false;
    }

    private boolean ContainsFlush()
    {
        Card.Suit suit = GetCardSuitAt(0);
        boolean isFlush = true;
        int index = 1;

        while(isFlush && index<GetNumCards())
        {
            if (suit != GetCardSuitAt(index))
            {
                isFlush = false;
            }
            else
            {
                index++;
            }
        }

        return isFlush;
    }

    private boolean ContainsRoyalFlush()
    {
        return ContainsFlush() && ContainsAce();
    }

    private boolean IsStraightHand()
    {
        boolean isStraight = true;
        int index = 1;

        while (isStraight && index < GetNumCards())
        {
            int currentCard = GetCardNumberAt(index);
            int previousCard = GetCardNumberAt(index - 1);

            if (!IsStillStraight(currentCard, previousCard))
            {
                isStraight = false;
            }
            else
            {
                index++;
            }
        }

        return isStraight;
    }

    private boolean IsStillStraight(int currentCard, int previousCard)
    {
        return currentCard == previousCard + 1;
    }

    private void OrderCardsByNumber()
    {
    	for(int i = 0; i <= cards.size(); i++){
    		for(int j = 0; j < cards.size() -1; j++){
    			if (cards.get(j).GetNumber() > cards.get(j+1).GetNumber()){
    				Card tmp = cards.get(j+1);
    				cards.set(j+1, cards.get(j));
    				cards.set(j,  tmp);
    			}
    		}
    	}    	
    }

    private boolean ContainsStraightFlush()
    {
        return ContainsFlush() && IsStraightHand();
    }

    private boolean ContainsRoyalStraightFlush()
    {
        return ContainsStraightFlush() && ContainsAce();
    }

    private boolean ContainsFourOfAKind()
    {
        return GetRepeatedCard(4) != GetNoRepeatedCardValue();
    }

    private boolean ContainsThreeOfAKind()
    {
        return GetRepeatedCard(3) != GetNoRepeatedCardValue();
    }

    private boolean ContainsPair()
    {
        return GetRepeatedCard(2) != GetNoRepeatedCardValue();
    }

    private boolean ContainsTwoPair()
    {
        Hand auxHand = new Hand(new ArrayList<Card>(GetCards()));
        int repeatedCard = auxHand.GetRepeatedCard(2);

        if (repeatedCard != GetNoRepeatedCardValue())
        {
            auxHand.RemoveCard(repeatedCard);
            auxHand.RemoveCard(repeatedCard);

            if (auxHand.GetRepeatedCard(2) != GetNoRepeatedCardValue())
            {
                return true;
            }
        }

        return false;
    }
}
