﻿import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

public class Tests
{
    /*TODO list
     * 1 7 8 4 5 -> High Card
     * 1 1 6 3 8 -> Pair
     * 2 2 10 10 5 -> Two Pair
     * 5 5 5 8 12 -> Three of a kind
     * 1 2 3 4 5 -> Straight
     * 2 3 7 5 9 (same suit) -> Flush
     * 1 1 4 4 4 -> Full House
     * 1 1 1 1 7 -> Poker
     * 1 2 3 4 5 (same suit) -> Straight Flush
     * 
     * */

    private Hand hand;

    @Test
    public void the_highest_card_is_the_best_combination(){
        createHand(
        		new Card(1, Card.Suit.Corazones),
                new Card(7, Card.Suit.Diamantes),
                new Card(8, Card.Suit.Picas),
                new Card(4, Card.Suit.Treboles),
                new Card(5, Card.Suit.Corazones)
        );

        Ranking bestRanking = FindBestRanking(hand);

        assertEquals(Ranking.HighCard, bestRanking);
    }

    @Test
    public void CheckPokerHandWithPair()
    {
        createHand(
                new Card(1, Card.Suit.Corazones),
                new Card(1, Card.Suit.Diamantes),
                new Card(6, Card.Suit.Picas),
                new Card(3, Card.Suit.Treboles),
                new Card(8, Card.Suit.Corazones)
        );

        Ranking result = FindBestRanking(hand); 
        assertEquals(Ranking.Pair, result);
    }

    @Test
    public void CheckPokerHandWithThreeOfAKind()
    {
        createHand(
                new Card(1, Card.Suit.Corazones),
                new Card(1, Card.Suit.Diamantes),
                new Card(6, Card.Suit.Picas),
                new Card(1, Card.Suit.Treboles),
                new Card(8, Card.Suit.Corazones)
            );

        Ranking result = FindBestRanking(hand);
        assertEquals(Ranking.ThreeOfAKind, result);
    }

    @Test
    public void CheckPokerHandWithTwoPair()
    {
        createHand(
                new Card(2, Card.Suit.Corazones),
                new Card(2, Card.Suit.Diamantes),
                new Card(10, Card.Suit.Picas),
                new Card(8, Card.Suit.Treboles),
                new Card(10, Card.Suit.Corazones)
        );

        Ranking result = FindBestRanking(hand);
        assertEquals(Ranking.TwoPair, result);
    }

    @Test
    public void CheckPokerHandWithFourOfAKind()
    {
        createHand(
                new Card(2, Card.Suit.Corazones),
                new Card(2, Card.Suit.Diamantes),
                new Card(10, Card.Suit.Picas),
                new Card(2, Card.Suit.Treboles),
                new Card(2, Card.Suit.Corazones)
        );

        Ranking result = FindBestRanking(hand);
        assertEquals(Ranking.FourOfAKind, result);
    }

    @Test
    public void CheckPokerHandWithStraight()
    {
        createHand(
                new Card(1, Card.Suit.Corazones),
                new Card(2, Card.Suit.Diamantes),
                new Card(3, Card.Suit.Picas),
                new Card(4, Card.Suit.Treboles),
                new Card(5, Card.Suit.Corazones)
            );
        
        Ranking result = FindBestRanking(hand);
        assertEquals(Ranking.Straight, result);
    }

    @Test
    public void CheckPokerHandWithFlush()
    {
        createHand(
                new Card(10, Card.Suit.Corazones),
                new Card(11, Card.Suit.Corazones),
                new Card(3, Card.Suit.Corazones),
                new Card(8, Card.Suit.Corazones),
                new Card(2, Card.Suit.Corazones)
            );

        Ranking result = FindBestRanking(hand);
        assertEquals(Ranking.Flush, result);
    }

    @Test
    public void CheckPokerHandWithStraightFlush()
    {
        createHand(
                new Card(6, Card.Suit.Corazones),
                new Card(4, Card.Suit.Corazones),
                new Card(3, Card.Suit.Corazones),
                new Card(2, Card.Suit.Corazones),
                new Card(5, Card.Suit.Corazones)
            );

        Ranking result = FindBestRanking(hand);
       
        assertEquals(Ranking.StraightFlush, result);
    }

    @Test
    public void CheckPokerHandWithFullHouse()
    {
        createHand(
                new Card(1, Card.Suit.Treboles),
                new Card(3, Card.Suit.Corazones),
                new Card(3, Card.Suit.Diamantes),
                new Card(1, Card.Suit.Treboles),
                new Card(3, Card.Suit.Picas)
            );

        Ranking result = FindBestRanking(hand);
        assertEquals(Ranking.FullHouse, result);
    }

    @Test
    public void CheckPokerHandWithRoyalFlush()
    {
        createHand(
                new Card(9, Card.Suit.Treboles),
                new Card(6, Card.Suit.Treboles),
                new Card(7, Card.Suit.Treboles),
                new Card(1, Card.Suit.Treboles),
                new Card(3, Card.Suit.Treboles)
            );

        Ranking result = FindBestRanking(hand);
        assertEquals(Ranking.RoyalFlush, result);
    }

    @Test
    public void CheckPokerHandWithRoyalStraightFlush()
    {
        createHand(
                new Card(5, Card.Suit.Diamantes),
                new Card(2, Card.Suit.Diamantes),
                new Card(4, Card.Suit.Diamantes),
                new Card(1, Card.Suit.Diamantes),
                new Card(3, Card.Suit.Diamantes)
            );

        Ranking result = FindBestRanking(hand);
        assertEquals(Ranking.RoyalStraightFlush, result);
    }

    @Test
    public void ComparesTwoHandsWithTheFirstHandBetter()
    {
        createHand(
                new Card(1, Card.Suit.Corazones),
                new Card(4, Card.Suit.Diamantes),
                new Card(8, Card.Suit.Picas),
                new Card(4, Card.Suit.Treboles),
                new Card(5, Card.Suit.Corazones)
            );

        List<Card> cards2 = Arrays.asList(
                new Card(1, Card.Suit.Corazones),
                new Card(7, Card.Suit.Diamantes),
                new Card(8, Card.Suit.Picas),
                new Card(4, Card.Suit.Treboles),
                new Card(5, Card.Suit.Corazones)
            );

        Hand hand2 = new Hand(cards2);

        Winner winner = hand.PlayAgainst(hand2);
        assertEquals(Winner.First, winner);
    }

    @Test
    public void ComparesTwoHandsWithTheSecondHandBetter()
    {
        createHand(
                new Card(1, Card.Suit.Corazones),
                new Card(4, Card.Suit.Diamantes),
                new Card(8, Card.Suit.Picas),
                new Card(4, Card.Suit.Treboles),
                new Card(5, Card.Suit.Corazones)
            );

        List<Card> cards2 = Arrays.asList(
                new Card(1, Card.Suit.Corazones),
                new Card(3, Card.Suit.Diamantes),
                new Card(2, Card.Suit.Picas),
                new Card(4, Card.Suit.Treboles),
                new Card(5, Card.Suit.Corazones)
            );

        Hand hand2 = new Hand(cards2);

        Winner result = hand.PlayAgainst(hand2);
        assertEquals(Winner.Second, result);
    }

    @Test
    public void ComparesTwoHandsWithTieHands()
    {
        createHand(
                new Card(1, Card.Suit.Corazones),
                new Card(4, Card.Suit.Diamantes),
                new Card(8, Card.Suit.Picas),
                new Card(4, Card.Suit.Treboles),
                new Card(4, Card.Suit.Corazones)
            );

        List<Card> cards2 = Arrays.asList(
                new Card(4, Card.Suit.Corazones),
                new Card(3, Card.Suit.Diamantes),
                new Card(2, Card.Suit.Picas),
                new Card(4, Card.Suit.Treboles),
                new Card(4, Card.Suit.Corazones)
            );

        Hand hand2 = new Hand(cards2);
        
        Winner winner = hand.PlayAgainst(hand2);
        assertEquals(Winner.Tie, winner);
    }
    
    protected Hand createHand(Card...cards)
    {
    	hand = new Hand(Arrays.asList(cards));
    	return hand;
    }
    
    
    private Ranking FindBestRanking(Hand hand)
    {
        return hand.FindBestRanking();
    }

}

